from lxml.etree import parse

namespaceMap = {
    'mods':"http://www.loc.gov/mods/v3",
    'extra':"http://meresco.com/namespace/fields/extra",
    'meta':"http://meresco.com/namespace/harvester/meta",
}

def xpath(node, path):
    result = node.xpath(path, namespaceMap)
    return result and result[0] or ''

def nodeset(node, path):
    return node.xpath(path, namespaceMap)

def add(aDictionary, key, value):
    if value:
            aDictionary[key] = value

class Record(object):
    def createRecord(self, metadata):
       self.metadata = metadata
       self.record = {}
       add(self.record, 'authors', self._getAuthors())
       add(self.record, 'title', self._getTitle())
       add(self.record, 'genre', self._getGenre())
       add(self.record, 'date', self._getDate())
       add(self.record, 'source', self._getSource())
       self._handleIdentifiers()
       return self.record

    def getMods(self, metadata):
        return xpath(metadata, '//mods:mods')

    def _getAuthors(self):
        authors = ""
        for name in nodeset(self.metadata, '//mods:mods/mods:name'):
            for namePart in nodeset(name, 'mods:namePart/text()'):
                authors += namePart
                authors += ' '
            authors = authors.rstrip(' ')
            authors += ', '
        return authors.rstrip(' ,')

    def _getTitle(self):
        title = xpath(self.metadata, '//mods:mods/mods:titleInfo/mods:title/text()') or '[NO TITLE AVAILABLE]'
        return title

    def _getGenre(self):
        genre = xpath(self.metadata, '//mods:mods/mods:genre/text()')
        if genre and '/' in genre: genre = genre.split('/')[-1]
	return genre

    def _getDate(self):
        return xpath(self.metadata, '//mods:originInfo/mods:dateIssued/text()')

    def _getHostTitle(self):
        return xpath(self.metadata, '//mods:mods/mods:relatedItem/mods:titleInfo/mods:title/text()')

    def _getHostEditors(self):
        editors = ""
        for name in nodeset(self.metadata, '//mods:mods/mods:relatedItem/mods:name'):
	    if xpath(name, 'mods:role/mods:roleTerm/text()') == 'edt':
                for namePart in nodeset(name, 'mods:namePart/text()'):
                    editors += namePart
                    editors += ' '
                editors = editors.rstrip(' ')
                editors += '; '
	if editors: return editors.rstrip(';') + '(Ed.)'
	else: return ''
        
    def _getSource(self):
        editors = self._getHostEditors()
	title = self._getHostTitle()
	if editors: return editors + ' ' + '<i>' + self._getHostTitle() + '</i>'
	else: return title

    def _handleIdentifiers(self):
	identifiers = (nodeset(self.metadata, '//mods:identifier/text()'))
	if not identifiers: return
	for id in identifiers:
            idParts = id.split(':')
            ### urn:{issn|issbn}:<value>
            if len(idParts) < 3: break
            if idParts[1] == 'issn' or idParts[1] == 'ISSN':
                add(self.record, 'issn', idParts[2])
            if idParts[1] == 'isbn' or idParts == 'ISBN':
                add(self.record, 'isbn', idParts[2])
